<?php
/**
 * Plugin Name: 📖Fedya' s Related Posts Selector
 * Description: Добавя секция "Може да харесате още" към публикациите и позволява вмъкване на линкове в съдържанието
 * Version: 1.3
 * Author: Fedya Serafiev
 *Author URI: https://fedia.eu
 * License: GPL v2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */

if (!defined('ABSPATH')) {
    exit;
}

// Добавяне на скриптове и стилове за админ панела
function enqueue_related_posts_scripts($hook) {
    global $post;
    
    if ($hook == 'post-new.php' || $hook == 'post.php') {
        wp_enqueue_script('jquery-ui-autocomplete');
        wp_enqueue_style('related-posts-style', plugins_url('css/style.css', __FILE__));
        wp_enqueue_script('related-posts-script', plugins_url('js/script.js', __FILE__), array('jquery', 'jquery-ui-autocomplete'), '1.0', true);
        
        // Добавяне на скрипт за Gutenberg
        wp_enqueue_script(
            'related-posts-gutenberg',
            plugins_url('js/gutenberg.js', __FILE__),
            array('wp-blocks', 'wp-element', 'wp-editor', 'wp-components'),
            '1.0',
            true
        );
        
        wp_localize_script('related-posts-script', 'relatedPostsAdmin', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('related_posts_nonce')
        ));
    }
}
add_action('admin_enqueue_scripts', 'enqueue_related_posts_scripts');

// Регистрация на бутона за TinyMCE редактор
function register_related_posts_button($buttons) {
    array_push($buttons, 'related_posts');
    return $buttons;
}
add_filter('mce_buttons', 'register_related_posts_button');

// Регистрация на TinyMCE plugin
function register_related_posts_plugin($plugin_array) {
    $plugin_array['related_posts'] = plugins_url('js/tinymce-plugin.js', __FILE__);
    return $plugin_array;
}
add_filter('mce_external_plugins', 'register_related_posts_plugin');

// Добавяне на мета кутия
function add_related_posts_meta_box() {
    add_meta_box(
        'related_posts_box',
        'Може да харесате още',
        'related_posts_meta_box_callback',
        'post',
        'normal',
        'high'
    );
}
add_action('add_meta_boxes', 'add_related_posts_meta_box');

// Callback за мета кутията
function related_posts_meta_box_callback($post) {
    wp_nonce_field('related_posts_meta_box', 'related_posts_meta_box_nonce');
    
    $related_posts = get_post_meta($post->ID, '_related_posts', true);
    if (!is_array($related_posts)) {
        $related_posts = array();
    }
    
    ?>
    <div class="related-posts-meta-box">
        <p><strong>Добавете свързани статии</strong></p>
        
        <div class="search-box">
            <input type="text" id="post-search" placeholder="Търсете статии..." class="widefat">
        </div>
        
        <div class="selected-posts">
            <ul id="selected-posts-list">
                <?php
                foreach ($related_posts as $post_id) {
                    $post_item = get_post($post_id);
                    if ($post_item) {
                        echo sprintf(
                            '<li data-id="%d">
                                <span class="post-title">%s</span>
                                <button type="button" class="remove-post button">✕</button>
                                <input type="hidden" name="related_posts[]" value="%d">
                            </li>',
                            $post_id,
                            $post_item->post_title,
                            $post_id
                        );
                    }
                }
                ?>
            </ul>
        </div>
    </div>
    <?php
}

// Записване на свързани публикации при запазване на публикация
function save_related_posts($post_id) {
    if (!isset($_POST['related_posts_meta_box_nonce']) || !wp_verify_nonce($_POST['related_posts_meta_box_nonce'], 'related_posts_meta_box')) {
        return;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    if (isset($_POST['post_type']) && $_POST['post_type'] === 'post' && !current_user_can('edit_post', $post_id)) {
        return;
    }

    $related_posts = isset($_POST['related_posts']) ? array_map('intval', $_POST['related_posts']) : array();
    update_post_meta($post_id, '_related_posts', $related_posts);
}
add_action('save_post', 'save_related_posts');

// Добавяне на модалния прозорец
function add_related_posts_modal() {
    ?>
    <div id="related-posts-modal" style="display:none;">
        <div class="related-posts-modal-content">
            <h2>Изберете свързана статия</h2>
            <input type="text" id="modal-post-search" placeholder="Търсете статии...">
            <div id="modal-search-results"></div>
        </div>
    </div>
    <?php
}
add_action('admin_footer', 'add_related_posts_modal');

// AJAX търсене на публикации
function search_related_posts_callback() {
    check_ajax_referer('related_posts_nonce', 'nonce');
    
    $search = sanitize_text_field($_POST['search']);
    $current_post_id = isset($_POST['current_post_id']) ? intval($_POST['current_post_id']) : 0;
    
    $args = array(
        'post_type' => 'post',
        'posts_per_page' => 10,
        's' => $search,
        'post_status' => 'publish',
        'post__not_in' => array($current_post_id)
    );
    
    $posts = get_posts($args);
    
    $results = array();
    foreach ($posts as $post) {
        $results[] = array(
            'id' => $post->ID,
            'label' => $post->post_title,
            'value' => $post->post_title
        );
    }
    
    wp_send_json($results);
}
add_action('wp_ajax_search_related_posts', 'search_related_posts_callback');

// Shortcode за вмъкване на свързана статия
function related_post_shortcode($atts) {
    $atts = shortcode_atts(array(
        'id' => 0,
        'title' => ''
    ), $atts);
    
    $post_id = intval($atts['id']);
    
    if ($post_id > 0) {
        $post = get_post($post_id);
        if ($post) {
            return sprintf(
                '<div class="inline-related-post">
                    <p>📖 <strong>Прочетете още:</strong> <a href="%s" target="_blank">%s</a></p>
                </div>',
                get_permalink($post),
                $post->post_title
            );
        }
    }
    
    return '';
}
add_shortcode('related_post', 'related_post_shortcode');

// Създаване на CSS файл
function create_related_posts_css() {
    $css_dir = plugin_dir_path(__FILE__) . 'css';
    if (!file_exists($css_dir)) {
        mkdir($css_dir);
    }
    
    $css_content = '
    .related-posts-meta-box {
        padding: 15px;
    }
    
    .search-box {
        margin-bottom: 20px;
    }
    
    #selected-posts-list {
        margin: 0;
        padding: 0;
    }
    
    #selected-posts-list li {
        display: flex;
        align-items: center;
        padding: 8px 12px;
        background: #f9f9f9;
        margin-bottom: 5px;
        border: 1px solid #ddd;
        border-radius: 3px;
    }
    
    #selected-posts-list .post-title {
        flex-grow: 1;
    }
    
    #selected-posts-list .remove-post {
        padding: 0 8px;
        color: #a00;
        border: none;
        background: none;
        cursor: pointer;
    }
    
    .inline-related-post {
        margin: 20px 0;
        padding: 15px;
        background: #f9f9f9;
        border-left: 4px solid #0073aa;
        border-radius: 3px;
    }
    
    #related-posts-modal {
        position: fixed;
        top: 50%;
        left: 50%;
        transform: translate(-50%, -50%);
        background: white;
        padding: 20px;
        z-index: 160000;
        width: 500px;
        max-width: 90%;
        box-shadow: 0 5px 15px rgba(0,0,0,0.3);
        border-radius: 5px;
    }
    
    .related-posts-modal-overlay {
        position: fixed;
        top: 0;
        left: 0;
        right: 0;
        bottom: 0;
        background: rgba(0,0,0,0.7);
        z-index: 159999;
    }
    
    #modal-search-results {
        margin-top: 10px;
        max-height: 300px;
        overflow-y: auto;
    }
    
    #modal-search-results .result-item {
        padding: 8px 12px;
        cursor: pointer;
        border-bottom: 1px solid #eee;
    }
    
    #modal-search-results .result-item:hover {
        background: #f0f0f0;
    }
    
    #modal-post-search {
        width: 100%;
        padding: 8px;
        margin: 10px 0;
    }
    ';
    
    file_put_contents($css_dir . '/style.css', $css_content);
}

// Създаване на JavaScript файлове
function create_related_posts_js_files() {
    $js_dir = plugin_dir_path(__FILE__) . 'js';
    if (!file_exists($js_dir)) {
        mkdir($js_dir);
    }
    
    // Основен скрипт
    $main_js = '
    jQuery(document).ready(function($) {
        // Код за мета кутията
        $("#post-search").autocomplete({
            source: function(request, response) {
                $.post(relatedPostsAdmin.ajaxurl, {
                    action: "search_related_posts",
                    nonce: relatedPostsAdmin.nonce,
                    search: request.term,
                    current_post_id: $("#post_ID").val()
                }, function(data) {
                    response(data);
                });
            },
            minLength: 2,
            select: function(event, ui) {
                var postItem = `
                    <li data-id="${ui.item.id}">
                        <span class="post-title">${ui.item.label}</span>
                        <button type="button" class="remove-post button">✕</button>
                        <input type="hidden" name="related_posts[]" value="${ui.item.id}">
                    </li>
                `;
                $("#selected-posts-list").append(postItem);
                $(this).val("").blur();
                return false;
            }
        });
        
        $(document).on("click", ".remove-post", function() {
            $(this).closest("li").remove();
        });
        
        // Код за модалния прозорец
        function openModal() {
            $("body").append("<div class=\'related-posts-modal-overlay\'></div>");
            $("#related-posts-modal").show();
            $("#modal-post-search").focus();
        }
        
        function closeModal() {
            $("#related-posts-modal").hide();
            $(".related-posts-modal-overlay").remove();
            $("#modal-post-search").val("");
            $("#modal-search-results").empty();
        }
        
        $(document).on("click", ".related-posts-modal-overlay", closeModal);
        
        var searchTimeout;
        $("#modal-post-search").on("input", function() {
            clearTimeout(searchTimeout);
            var searchTerm = $(this).val();
            
            if (searchTerm.length < 2) {
                $("#modal-search-results").empty();
                return;
            }
            
            searchTimeout = setTimeout(function() {
                $.post(relatedPostsAdmin.ajaxurl, {
                    action: "search_related_posts",
                    nonce: relatedPostsAdmin.nonce,
                    search: searchTerm,
                    current_post_id: $("#post_ID").val()
                }, function(data) {
                    var results = "";
                    data.forEach(function(item) {
                        results += `<div class="result-item" data-id="${item.id}" data-title="${item.label}">
                            ${item.label}
                        </div>`;
                    });
                    $("#modal-search-results").html(results);
                });
            }, 300);
        });
        
        $(document).on("click", ".result-item", function() {
            var postId = $(this).data("id");
            var postTitle = $(this).data("title");
            var shortcode = `[related_post id="${postId}" title="${postTitle}"]`;
            
            if (typeof wp !== "undefined" && wp.data && wp.data.select("core/editor")) {
                // Gutenberg
                var block = wp.blocks.createBlock("core/shortcode", {
                    text: shortcode
                });
                wp.data.dispatch("core/editor").insertBlocks(block);
            } else {
                // Classic Editor
                window.send_to_editor(shortcode);
            }
            
            closeModal();
        });
        
        // Добавяне на бутон в класическия редактор
        if (typeof QTags !== "undefined") {
            QTags.addButton("related_post", "Вмъкни свързана статия", function() {
                openModal();
            });
        }
    });
    ';
    file_put_contents($js_dir . '/script.js', $main_js);
    
    // TinyMCE plugin
    $tinymce_js = '
    (function() {
        tinymce.create("tinymce.plugins.RelatedPosts", {
            init: function(editor, url) {
                editor.addButton("related_posts", {
                    title: "Вмъкни свързана статия",
                    icon: "link",
                    onclick: function() {
                        jQuery("#related-posts-modal").show();
                        jQuery("body").append("<div class=\'related-posts-modal-overlay\'></div>");
                    }
                });
            }
        });
        
        tinymce.PluginManager.add("related_posts", tinymce.plugins.RelatedPosts);
    })();
    ';
    file_put_contents($js_dir . '/tinymce-plugin.js', $tinymce_js);
    
    // Gutenberg интеграция
    $gutenberg_js = '
(function(wp) {
    var registerPlugin = wp.plugins.registerPlugin;
    var PluginSidebar = wp.editPost.PluginSidebar;
    var el = wp.element.createElement;
    var registerBlockType = wp.blocks.registerBlockType;
    var Button = wp.components.Button;
    var toolbar = wp.blockEditor.BlockControls;
    
    // Register a toolbar button for inserting related posts
    registerPlugin("related-posts-button", {
        render: function() {
            return el(
                toolbar,
                null,
                el(
                    Button,
                    {
                        icon: "admin-links",
                        label: "Вмъкни свързана статия",
                        onClick: function() {
                            jQuery("#related-posts-modal").show();
                            jQuery("body").append("<div class=\'related-posts-modal-overlay\'></div>");
                        }
                    }
                )
            );
        }
    });
    
    // Register a custom block for related posts
    registerBlockType("related-posts/inline-link", {
        title: "Свързана статия",
        icon: "admin-links",
        category: "common",
        attributes: {
            postId: {
                type: "number",
                default: 0
            },
            postTitle: {
                type: "string",
                default: ""
            }
        },
        
        edit: function(props) {
            var postId = props.attributes.postId;
            var postTitle = props.attributes.postTitle;
            
            return el(
                "div",
                { className: "inline-related-post" },
                el(
                    "p",
                    null,
                    "📖 ",
                    el("strong", null, "Прочетете още: "),
                    postId ? postTitle : "Изберете свързана статия",
                    el(
                        Button,
                        {
                            isSecondary: true,
                            onClick: function() {
                                jQuery("#related-posts-modal").show();
                                jQuery("body").append("<div class=\'related-posts-modal-overlay\'></div>");
                                
                                // Add listener for selection
                                jQuery(document).one("click", ".result-item", function() {
                                    var selectedId = jQuery(this).data("id");
                                    var selectedTitle = jQuery(this).data("title");
                                    
                                    props.setAttributes({
                                        postId: selectedId,
                                        postTitle: selectedTitle
                                    });
                                });
                            },
                            style: { marginLeft: "10px" }
                        },
                        "Избери статия"
                    )
                )
            );
        },
        
        save: function(props) {
            var postId = props.attributes.postId;
            
            // Return the shortcode as content
            return "[related_post id=\"" + postId + "\" title=\"" + props.attributes.postTitle + "\"]";
        }
    });
})(window.wp);
';
    file_put_contents($js_dir . '/gutenberg.js', $gutenberg_js);
}

// Създаване на файлове при активация на плъгина
function create_related_posts_files() {
    create_related_posts_css();
    create_related_posts_js_files();
}
register_activation_hook(__FILE__, 'create_related_posts_files');
?>